import java.io.*;
import java.net.*;
import java.util.*;

class BCHostSim extends Thread
{
	private Socket localConnect;
	private DataInputStream in;
	private DataOutputStream out;

	public static void main(String[] args)
	{
		int listenport;
		ServerSocket ss;

		if ( args.length != 1) {
			System.err.println("Usgage: java BCHostSim port");
			System.exit(1);
		}

		listenport = Integer.parseInt( args[0]);

		try {
			ss = new ServerSocket(listenport);
			while (true) {
				BCHostSim bchost = new BCHostSim(ss.accept());
				bchost.start();
			}
		} catch (IOException e) { 
			System.out.println("Listen Exception " + e);	
		}
	}

	public BCHostSim(Socket s)
	{
		localConnect = s;

	}

	//
	// BCComData Head б
	//	
	BCComData recvBCComData()
	{
		byte[] buffer = new byte[1024];
		for (int i = 0; i < 1024; i ++) buffer[i] = (byte) ' ';

		BCComData bcdata= null;
		int	datalen;

		try {
			in.readFully(buffer,0,BCComDef.BCComDataHeadSize);
			bcdata = BCComData.toObject(buffer);
			if (bcdata == null) return null;
			bcdata.setSize(0);

		} catch (InterruptedIOException ex) {
			return null;	
		} catch (IOException ex){
			return null;
		}

		System.out.println("RECV From Client<<==");
		BCComUtil.hexaDump(System.out,bcdata.toByte(),
					BCComDef.BCComDataHeadSize+bcdata.getSize());

		return bcdata;
	}

	//
	// BCComData Data б
	//	
	BCComData recvBCComData(BCComData bccom,int offset,int length) 
	{
		byte[] buffer = new byte[1024];
		for (int i = 0; i < 1024; i ++) buffer[i] = (byte) ' ';

		try 
		{
			in.readFully(buffer,0,length);
			bccom.setData(offset,buffer,0,length);
			bccom.setSize(offset+length);
		} catch (InterruptedIOException ex) {
			return null;	
		} catch (IOException ex){
			return null;
		}
		System.out.println("RECV From Client<<==");
		BCComUtil.hexaDump(System.out,bccom.toByte(),
					BCComDef.BCComDataHeadSize+bccom.getSize());
		return bccom;
	}


	boolean sendBCComData(BCComData bcdata)
	{
		byte[] buffer = bcdata.toByte();

		System.out.println("SEND To Client==>>");
		BCComUtil.hexaDump(System.out,buffer,
					BCComDef.BCComDataHeadSize+bcdata.getSize());
		try {
			out.write(buffer,0,BCComDef.BCComDataHeadSize+bcdata.getSize());
		} catch (IOException ex) {
			return false;	
		}
		return true;
	}


	//
	// HostSim Main
	//
	public void run()
	{
		BCComData bcdata;

		//##1 Set Socket Reading Timeout
		try { 
			localConnect.setSoTimeout(1000*300000);
		} catch (SocketException ex) {
			System.out.println("SetSoTimeout Error!!");
			return;
		}

		//##2 Construct In/Out Stream
		try {
			in  = 
					new DataInputStream(localConnect.getInputStream());
			out = 
					new DataOutputStream(localConnect.getOutputStream());
		} catch (IOException iex) {
			System.out.println("Get Strem Error!!");
			return;
		}

	while(true)
	{
		//##3 Read & Construct BCComData Head
		bcdata = recvBCComData();
		if (bcdata == null) {
			System.out.println("Client Read Error!!");
			try { localConnect.close();} catch(IOException iex) { };
			return;
		}

		//##4  ޽ б ó

		// ȸϰȸ û
		if ( bcdata.getMsgCode().compareTo("0440") == 0 ) 		
		{
			recvReqInquiry(bcdata);
		} 

 		// KIBBȸ / û
		else if (bcdata.getMsgCode().compareTo("0450") == 0 )
		{
			recvReqEnroll(bcdata);
		} 
		
		// ޽  
		else 												
		{
			System.out.println("Msg[" + bcdata.getMsgCode() + "]");
		}
	}
	}


	// Msg Receive Handler
	void recvReqInquiry(BCComData bccom)
	{
		BCComData tmpBccom;
		tmpBccom = recvBCComData(bccom,0,14);
		if (tmpBccom  == null ) 
		{
			System.out.println("ReqInquiry(0400) Data Read Error");
			return;
		}

		byte[] buffer;

		String rspInquiry = "01" 
						  + "211645551887"
						  + "                      " 
						  + "000"
						  + "                ";

		buffer = rspInquiry.getBytes();

		tmpBccom.setData(14,buffer,0,buffer.length);
		tmpBccom.setSize(926);
		tmpBccom.setMsgCode("0441");
		System.out.println("SET TRANCE  ");
		BCComUtil.hexaDump(System.out,tmpBccom.toByte(),
					BCComDef.BCComDataHeadSize+tmpBccom.getSize());

		if ( sendBCComData(tmpBccom) != true ) {
			System.out.println("Client Write Error!!");
			try { localConnect.close();} catch(IOException iex) { };
			return;
		}

	}

	void recvReqEnroll(BCComData bccom)
	{
		BCComData tmpBccom;
		tmpBccom = recvBCComData(bccom,0,940);
		if (tmpBccom  == null ) 
		{
			System.out.println("ReqEnroll(0450) Data Read Error");
			return;
		}

/**
**/
		byte[] buffer;

		String rspInquiry = "01" 
						  + "211645551887"
						  + "                      " 
						  + "000"
						  + "                ";

		buffer = rspInquiry.getBytes();

		tmpBccom.setData(14,buffer,0,buffer.length);
		tmpBccom.setSize(940);
		tmpBccom.setMsgCode("0451");
		System.out.println("SET TRANCE  ");
		BCComUtil.hexaDump(System.out,tmpBccom.toByte(),
					BCComDef.BCComDataHeadSize+tmpBccom.getSize());

		if ( sendBCComData(tmpBccom) != true ) {
			System.out.println("Client Write Error!!");
			try { localConnect.close();} catch(IOException iex) { };
			return;
		}

	}

	// Msg Sender Handler
}
