import java.io.*;
import java.net.*;


/**
 * 
 *
 *
 */
class InfoMgr
{
	RIFTab riftab;
	IFTab  iftab;
	LogMgr log;

	int   recentlyUsedUpperLine;
	
	/**
	 * InforMgr no-arg Constructor
	 *
	 */
	InfoMgr()
	{
		riftab = new RIFTab();
		iftab  = new IFTab();	
		log    = new LogMgr();
		recentlyUsedUpperLine = 0;
	}
	
	/**
	 * Find a empty slot in IF Table.
	 */
	int findIFTabEmptySlot()
	{
		int ret = -1;
		
		synchronized(iftab) 
		{	
			for (int i = 0;i <1024; i++) {
				if ( iftab.en[i].onuse == false) {
					iftab.en[i].init();
					iftab.en[i].onuse = true;
					return i;
				}
			}
			ret = -1;	
		}
		
		return ret;
	}

	/**
	 * Find a empty slot in RIF Table.
	 */
	int findRIFTabEmptySlot()
	{
		int ret = -1;
		
		synchronized(riftab) 
		{	
			for (int i = 0;i <10; i++) {
				if ( riftab.en[i].onuse == false) {
					riftab.en[i].init();
					riftab.en[i].onuse = true;
					return i;
				}
			}
			ret = -1;	
		}
		
		return ret;
	}
	
	/**
	 * Find the slot matched to a given transaction sequence
	 * @param : String transSeq : Transaction Sequence No. <BR>
	 * @return int : IF Table Index matched to a given transaction sequence if found<BR>
	 *               -1 if not found<BR>
	 */
	private int findIFTabSlotByTransSeq(String transSeq)
	{
		if (transSeq == null ) {
			log.writeLog("[RIF] transSeq is null,so return -1");
			return -1;
		}
		
		for (int i=0;i <1024; i++) {
			if ( iftab.en[i].onuse == true && iftab.en[i].transSeq != null ) 
			{
				if ( iftab.en[i].transSeq.compareTo(transSeq) ==0 ) 
					return i;
			}
		}
		return -1;
	}
	
	/*
	 * Remove IF Table Slot indexed by nrifID
	 *
	 */
	void rmIFTabSlot(int nrifID)
	{
		synchronized (iftab) 
		{
			iftab.en[nrifID].init();	
		}
	}	


	/*
	 * Close a RIF socket connection then clear that slot
	 *
	 */
	void discRIFConn(int nrifID) 
	{
		// Check Index
		if (nrifID < 0 || nrifID >= 10) 
		{
			log.writeLog("[RIF] nrifID Index Error ID[" + nrifID + "]");
			return;
		}

		synchronized(iftab) {
			try { 
				riftab.en[nrifID].connect.close();
			} catch (IOException ex) {
				log.writeLog("[RIF] Close Error  ID["+nrifID+"]");
			}
			riftab.en[nrifID].isConnect = false;
		}
	}

	/*
	 * Close a IF socket connection then clear that slot. 
	 * if that slot have a sequence number, write a error log 
	 * @param : int nrifID - IF Table index
	 */
	void discIFConn(int nrifID) {

		// Check Index
		if (nrifID < 0 || nrifID >= 1024) 
		{
			log.writeLog("[IF] nrifID Index Error ID[" + nrifID + "]");
			return;
		}

		// Response Request
		if (iftab.en[nrifID].transSeq != null ) 
		{
			log.writeLog("[IF] Response Error Seq["+iftab.en[nrifID].transSeq +
						 "] ID[" + nrifID + "]");

		}
			
		synchronized(iftab) {
			try { 
				iftab.en[nrifID].connect.close();
			} catch (IOException ex) {
				log.writeLog("[IF] Close Error  ID["+nrifID+"]");
			}
			iftab.en[nrifID].init();
		}
	}

	
	/*
	 * Receive a packet from a upper connection, 
	 * then transform that packet into BCData.
	 * 
	 * @param : BCComData bcdata - BC packet Data, out parameter
	 * @return int : >0 number of byte read if succeed in translation
	 *        		 0  if fail to translate
	 *               <0 if occuered a error while reading from socket
	 */
	int recvUpper(BCComData bcdata,int nrifID)
	{
		int len=0;
		int ret = -1;
		byte[] buffer  = new byte[1024];

		for (int i = 0; i < 1024; i ++) buffer[i] = (byte) ' ';
		bcdata.init();
	
		synchronized(riftab.en[nrifID].in) {
			len = 0;
			ret = 0;
			try {
				len = riftab.en[nrifID].in.read(buffer,0,1024);
			} catch (IOException ex) {
				ret = -1;
			}
		}
		
		//  DISC
		if (len < 0 || ret < 0) {
			log.writeLog("[IF]DISC");
			return -1;
		}	

		// Size Error
		if (len < BCComDef.BCComDataHeadSize) {
			log.log(buffer,len,"[RIF]RECV from Server["+ nrifID +"] Size Error(" + len + ")");
			return 0;
		}
		
		if (bcdata.toObject(buffer,len) != true) {
			log.log(buffer,len,"[RIF]RECV from Server" + nrifID + "] Convert Error(" + len + ")");
			return -1;
		}
	
		bcdata.setSize(len-BCComDef.BCComDataHeadSize);
		log.log(buffer,len,"[RIF]RECV from Server[" + nrifID + "]");
			
		return len;
	}


	synchronized int findNextUpperLine()
	{
		int i = 0;
		int end = 0;
		int findIdx = -1;

		i = (recentlyUsedUpperLine +1)%10;

		// ش   α
		while( i != recentlyUsedUpperLine )
		{
			if ( riftab.en[i].isConnect == true &&
				 riftab.en[i].onuse == true ) {
				findIdx = i;
				recentlyUsedUpperLine = i;
				break;
			}
			i = (i+1)%10;
		} 

		// ٸ  ٸ ֱٻ  ̿밡 checkϿ
		// ̿밡ϸ   Ѵ.
		if (findIdx < 0 ) {
			if (riftab.en[i].isConnect == true &&
				riftab.en[i].onuse == true ) 
			{
				findIdx = recentlyUsedUpperLine;
			}
		}

		return findIdx;
	}

	synchronized boolean isUpperConnect()
	{
		boolean found;

		found = false;
		for (int i =0; i < 10; i ++) {
			if (riftab.en[i].onuse == true && 
				riftab.en[i].isConnect == true ) 
			{
				found = true;
				break;
			}
		}

		return found;
	}

	/*
	 * Send a BCComData to upper connection indexed by nrifID
	 */
	boolean sendUpper(BCComData bcdata)
	{
		int nrifID;
		boolean ret;
		byte[] buffer = bcdata.toByte();		
		int len = BCComDef.BCComDataHeadSize+bcdata.getSize();

		//    
		nrifID = findNextUpperLine();
		if (nrifID < 0) {
			log.log(buffer,len,"[IF]Connection:Ÿ");
			return false;
		}
		log.writeLog("[IF] Send Uppler Line[" +nrifID+ "]");
		
		if (riftab.en[nrifID].isConnect != true ) 
		{	
			log.log(buffer,len,"[IF]:Ÿ");
			return false;
		} 

		//## debug -s
		//   for converting 0x00 to ' ' 
		for (int i=0;i < len; i++ ) 
		{
			if (buffer[i] == (byte) 0x00) 
			{
				buffer[i] = (byte)' ';	
			}
		}
		//## debig -e
		

		synchronized(riftab.en[nrifID].out) {
			try {
				riftab.en[nrifID].out.write(buffer,0,len);
				//## [2000.11.27] packetizing  -S
				//packetizing problem packet   flush
				//test  ذå .
				riftab.en[nrifID].out.flush();
				//## [2000.11.27] packetizing  -S
			} catch (IOException ex) {
				log.log(buffer,len, "[IF] send to Server[" + nrifID + "]fail!!" );
				return false;
			}
		}
		log.log(buffer,len, "[IF] send to Server[" + nrifID + "] SUCC!!" );
		
		return true;
	}
	
	
	int recvLow(BCComData bcdata, int nrifID)
	{
		int len=0;
		int ret = -1;
		byte[] buffer  = new byte[1024];

		
		for (int i = 0; i < 1024; i ++) buffer[i] = (byte) ' ';
		bcdata.init();
	
		synchronized(iftab.en[nrifID].in) {
			ret = 0;
			len = 0;
			try {
				len = iftab.en[nrifID].in.read(buffer,0,1024);
			} catch (IOException ex) {
				log.writeLog("[IF]DISC ID[" +nrifID+ "]");
				ret = -1;
			}
		}
		
		// ϴ DISC
		if (len < 0 || ret < 0) {
			log.writeLog("[IF]DISC ID[" +nrifID+ "]");
			return -1;
		}	

		// Size Error
		if (len < BCComDef.BCComDataHeadSize) {
			log.log(buffer,len,"[IF]RECV from Client ID[" +nrifID+ "] Size Error(" + len + ")");
			return 0;
		}
		
		if (bcdata.toObject(buffer,len) != true) {
			log.log(buffer,len,"[IF]RECV from Client ID[" +nrifID+ "] Convert Error(" + len + ")");
			return -1;
		}
	
		bcdata.setSize(len-BCComDef.BCComDataHeadSize);
		log.log(buffer,len,"[IF]RECV from client ID[" + nrifID + "]");
			
		return len;
	}



	int sendLow(BCComData bcdata)
	{
		int nrifID = 0;
		
		byte[] buffer = bcdata.toByte();
		int len = BCComDef.BCComDataHeadSize+bcdata.getSize();

		synchronized(iftab)  {
			nrifID = findIFTabSlotByTransSeq(bcdata.getTransSeq());
			if (nrifID < 0) 
			{
				
				log.log(buffer,len,"[RIF] Send Client NG No Routing Info Seq["+
						bcdata.getTransSeq() + "]");	
				return -1;
			} 
			else 
			{
				iftab.en[nrifID].transSeq = null;
				try 
				{
					iftab.en[nrifID].out.write(buffer,0,len);
				} catch (IOException ex) {
					log.log(buffer,len, "[RIF] Send Client NG Seq["+
						   bcdata.getTransSeq() + "] ID[" + nrifID + "]");
					return -1;
				}
			} //end if
		}
		log.log(buffer,BCComDef.BCComDataHeadSize+bcdata.getSize(),
  	                "[RIF] Send Client OK Seq[" + bcdata.getTransSeq() + 
				     "] ID[" + nrifID + "]");
		
		return nrifID;
	}



	//
	//
	//
	private int recvBCComData ( BCComData bcdata, DataInputStream in )
	{	
		
		byte[] buffer  = new byte[1024];
		int len;

		
		for (int i = 0; i < 1024; i ++) buffer[i] = (byte) ' ';
		bcdata.init();
		
		try {
			len = in.read(buffer,0,1024);
		} catch (IOException ex) {
			return -1;
		}
		if (len < 0) {
			return -1;
		}	

		log.dumpData(buffer,len);

		if (len < BCComDef.BCComDataHeadSize) {
			return 0;
		}
		
		boolean ret = bcdata.toObject(buffer,len);
		if (ret == false) return -1;
		bcdata.setSize(len-BCComDef.BCComDataHeadSize);

		return len;
	}


	private boolean sendBCComData(BCComData bcdata, DataOutputStream out)
	{
		byte[] buffer = bcdata.toByte();

		log.dumpData(buffer,BCComDef.BCComDataHeadSize+bcdata.getSize());

		try {
			out.write(buffer,0,BCComDef.BCComDataHeadSize+bcdata.getSize());	
		} catch (IOException ex) {
			return false;
		}
		
		return true;	
	}
}
